; VGM player for the NeoGeo Pocket
; Z80 VGM playback driver
;
; v2: The difference in LFSR width (15 v 16 bits) is
;     compensated for, but tone channel pitches are
;     not corrected, so the whole song will sound
;     sound slightly out of tune, but all channels
;     should be out of tune by the same amount.
;
; /Mic, 2012


.MEMORYMAP
	DEFAULTSLOT 0
	SLOTSIZE $1000
	SLOT 0 $0000
.ENDME

.ROMBANKSIZE $1000
.ROMBANKS 1
.BANK 0 SLOT 0
.ORGA $00


; Sound registers
;
.define REG_CH4 $4000
.define REG_CH123 $4001


; Circular buffer of 256 bytes containing VGM data
; The TLCS900H reads the VGM data from ROM and writes it
; to the corresponding shared RAM area on its side, and the
; Z80 driver then consumes the data.
;
.define VGM_DATA_BUFFER $0C00

.define VGM_BUFFER_READ_POS $8000

.define CH1_PER $0D00
.define CH2_PER $0D02
.define CH3_PER $0D04
.define CH4_PER $0D06
.define CUR_CHN $0D08
.define VGM_RESTART $0D0A
.define SAMPLE_DRIFT $0D0B
.define NOISE_CFG $0D0D
.define CH3_PER2 $0D0E

; VGM commands recognized by the driver
;
.define CMD_GG_STEREO	$4F
.define CMD_PSG		$50
.define CMD_FM		$51
.define CMD_WAIT	$61
.define CMD_WAIT_FRAME_NTSC	$62
.define CMD_WAIT_FRAME_PAL	$63
.define CMD_END		$66


di
im 	1
jp 	vgm_play


; IRQ handler
.orga $0038
	reti	

; NMI handler	
.orga $0066
	retn


.orga $0100

vgm_play:  
        ld	hl,0
        ld	a,0
        ld	(SAMPLE_DRIFT),hl
        ld	(CH1_PER),hl
        ld	(CH2_PER),hl
        ld	(CH3_PER),hl
        ld	(CUR_CHN),a
        ld	(NOISE_CFG),a
        
	ld	bc,VGM_DATA_BUFFER
	ld	iy,vgm_jump_table
	
vgm_play_loop:
	ld	a,c			; 4
	ld	(VGM_BUFFER_READ_POS),a	; 13
	ld	a,(bc)			; 7. Read one byte from the VGM data buffer
	inc	c			; 4
	ld	ixh,a			; 8. Save the VGM command in IXH
    	add     a,$C0 			; 7. subtract 0x40
    	rlca            		; 4. multiply by 2
    	ld      iyl,a   		; 8
    	ld      l,(iy+0) 		; 19. Load HL with command handler address
    	ld      h,(iy+1) 		; 19
    	jp      (hl)    		; 4. Jump to the handler (tot 97 cycles)
	


vgm_psg_write:
	ld 	a,(bc)			; 7
	inc	c			; 4
	ld	ixh,a			; 8. Save a
	bit	7,a			; 8
	jp	z,data_byte		; 10 

	and	$F0			; 7. 1cct0000
        ld 	(CUR_CHN+1),a
	ld	a,ixh
	
	bit	4,a			; 8
	jp	nz,psg_volume		; 10 
	rlca				; 4. cctdddd1
	rlca				; 4. ctdddd1c
	rlca				; 4. tdddd1cc
	and	3			; 7 000000cc
	rlca				; 4
	ld	(CUR_CHN),a		; 13
	cp	3*2			; 7
	jp	z,psg_noise		; 10 
	ld	l,a			; 4
	ld	h,>CH1_PER		; 7
	ld	a,ixh			; 8
	and	$0F			; 7. 0000dddd
	ld	d,a			; 4
	ld	a,(hl)			; 7. DDDDdddd
	and	$F0			; 7
	or	d			; 4 
	ld	(hl),a			; 7
	ld	e,a			; 4. DDDDdddd

	ld	a,(CUR_CHN)
	cp	2*2
	jr	nz,psg_write
	ld	a,(NOISE_CFG)
	cp	3
	jr	nz,psg_write
	
	ld	a,ixh
	ld	(CH3_PER2),a
	
	inc	hl			; 6
	ld	d,(hl)			; 7. DE = 000000DD DDDDdddd
	ld	hl,sms2ngp_freqlo_tbl	; 10
	add	hl,de			; 11 
	ld	d,(hl)			; 7
	ld	a,(CUR_CHN+1)		; 13
	or	d			; 4. 1cctdddd
	;;ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13
	ld	a,(SAMPLE_DRIFT)	; 13
	;add	a,7			; 7
	inc a
	ld	(SAMPLE_DRIFT),a	; 13
	inc 	h			; 4
	inc 	h			; 4
	inc 	h			; 4
	inc 	h			; 4
	ld 	a,(hl)			; 7
	;;ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13
	;;jp	vgm_play_loop		; 10 
psg_write:
	ld	a,ixh
	ld	(REG_CH123),a		; 13
	;ld	(REG_CH4),a		; 13
	ld	a,(SAMPLE_DRIFT)	; 13
	add	a,6			; 7
	ld	(SAMPLE_DRIFT),a	; 13
	jp	vgm_play_loop		; 10 
	
data_byte:
	ld 	a,(CUR_CHN+1)		; 13
	bit	4,a
	jp	nz,psg_volume
	ld	a,(CUR_CHN)		; 13
	cp	3*2			; 7
	jp	z,psg_noise		; 10 
	ld	l,a			; 4
	ld	h,>CH1_PER		; 7
	ld	a,ixh			; 8
	and	$3F			; 7. 00DDDDDD
	ld	e,a			; 4 (121, 224)
	ld	a,0			; 7
	sla	e			; 8		
	rla				; 4. AE = 00000000 0DDDDDD0		
	sla	e			; 8
	rla				; 4. AE = 00000000 DDDDDD00	
	sla	e			; 8
	rla				; 4. AE = 0000000D DDDDD000
	sla	e			; 8
	rla				; 4 (176, 279). AE = 000000DD DDDD0000
	inc	hl			; 6
	ld	(hl),a			; 7. CHx_PER = (CHx_PER & 0xFF) | (A << 8)
	ld	d,a			; 4. D = 000000DD
	dec	hl			; 6
	ld	a,(hl)			; 7. DDDDdddd
	and	$0F			; 7. 0000dddd
	or	e			; 4. A = DDDDdddd
	ld	e,a			; 4. DE = 000000DD DDDDdddd
	ld	(hl),a			; 7. CHx_PER = (CHx_PER & 0xFF00) | A

	ld	a,(CUR_CHN)
	cp	2*2
	jr	nz,psg_write
	ld	a,(NOISE_CFG)
	cp	3
	jr	nz,psg_write
	
	ld	a,ixh
	ld	(CH3_PER2+1),a
	
	ld	hl,sms2ngp_freqlo_tbl	; 10
	add	hl,de			; 11 

	ld 	d,(hl)			; 7
	ld	a,(CUR_CHN+1)		; 13
	or	d			; 4
	ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13
	inc 	h			; 4
	inc 	h			; 4
	inc 	h			; 4
	inc 	h			; 4

	ld	a,(SAMPLE_DRIFT)	; 13
	;add	a,7			; 7
	inc a
	ld	(SAMPLE_DRIFT),a	; 13
	
	ld	a,(hl)			; 7. 00DDDDDD
	ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13

	jp	psg_write ;vgm_play_loop		; 10 	
psg_noise:
	ld	a,ixh			; 8
	;ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13

	ld	d,a
	ld	a,(NOISE_CFG)
	ld	e,a
	ld	a,d
	and	3			; 7
	ld	(NOISE_CFG),a		; 13
	cp	3
	jr	nz,+
	ld	hl,(CH3_PER)		; 16
	ld	de,sms2ngp_freqlo_tbl	; 10
	add	hl,de			; 11 
	ld 	a,(hl)			; 7
	or	$C0			; 7
	;ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13
	inc 	h			; 4
	inc 	h			; 4
	inc 	h			; 4
	inc 	h			; 4
	ld	a,(SAMPLE_DRIFT)	; 13
	inc	a			; 4
	ld	(SAMPLE_DRIFT),a	; 13
	ld	a,(hl)			; 7. 00DDDDDD
	;ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13
	ld	a,(SAMPLE_DRIFT)	; 13
	add	a,4			; 7
	ld	(SAMPLE_DRIFT),a	; 13
	jp	vgm_play_loop		; 10 
+:
	ld	a,e
	cp	3
	jr	nz,+
	;ld	a,(CH3_PER2)
	;ld	(REG_CH123),a		; 13
	;ld	(REG_CH4),a		; 13
	;ld	a,(CH3_PER2+1)
	;ld	(REG_CH123),a		; 13
	;ld	(REG_CH4),a		; 13
+:
	ld	a,(SAMPLE_DRIFT)	; 13
	add	a,4			; 7
	ld	(SAMPLE_DRIFT),a	; 13
	jp	vgm_play_loop		; 10 
psg_volume:
	ld	(REG_CH123),a		; 13
	ld	(REG_CH4),a		; 13

	ld	a,(SAMPLE_DRIFT)	; 13
	add	a,3			; 7
	ld	(SAMPLE_DRIFT),a	; 13
	jp	vgm_play_loop		; 10 

	
vgm_fm_write:	; Ignored
	inc	c
	inc	c
	jp	vgm_play_loop
	
	
vgm_gg_stereo:	; Ignored
	inc	c
	jp	vgm_play_loop


vgm_end:
	ld	a,1
	ld	(VGM_RESTART),a
-:
	ld	a,(VGM_RESTART)
	and	a
	jr	nz,-
	jp	vgm_play_loop



vgm_short_wait:
	ld	a,ixh	; 8
	and	$0F	    ; 7
	jp	z,vgm_play_loop	; 10
	
	; 69 cycles/iteration
-:
	ld	ix,(0)	; 20
	ld	ix,(0)	; 20
	inc	ixh	; 8
	and	$0F	; 7
	dec	a	; 4
	jp	nz,-	; 10
	jp	vgm_play_loop ; 10


vgm_wait:
	ld	a,(bc)
	inc	bc
	ld	b,>VGM_DATA_BUFFER
	ld	e,a
	ld	a,(bc)
	inc	bc
	ld	b,>VGM_DATA_BUFFER
	ld	d,a
vgm_wait2:
	ld	hl,(SAMPLE_DRIFT) ; 16
	ex	de,hl		; 4
	scf			; 4
	sbc	hl,de		; 15	
	dec 	hl		; 6 
	dec	hl		; 6. wait - SAMPLE_DRIFT - 3
	ex	de,hl		; 4
	
	; 69 cycles/iteration
-:
	ld	ix,(0)		; 20
	jp	+		; 10
+:	ld	a,ixh		; 8
	dec	de		; 6
	and	$0F	    	; 7
	ld	a,e		; 4
	or	d		; 4
	jp	nz,-		; 10
	
	ld	hl,0		; 10
	ld	(SAMPLE_DRIFT),hl ; 16 
	jp	vgm_play_loop 	; 10
	
	
vgm_wait_pal:
	ld	de,882		; 10
	jp	vgm_wait2	; 10
	
vgm_wait_ntsc:
	ld	de,735		; 10
	jp	vgm_wait2	; 10



.ORGA $300

; Needs to be aligned on a 256-byte boundary
vgm_jump_table:
.dw 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,vgm_gg_stereo
.dw vgm_psg_write,vgm_fm_write,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0
.dw 0,vgm_wait,vgm_wait_ntsc,vgm_wait_pal, 0,0,vgm_end,0, 0,0,0,0, 0,0,0,0
.dw vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait
.dw vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait,vgm_short_wait


.ORGA $400

sms2ngp_freqlo_tbl:
.db $01,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00
.db $01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01
.db $02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02
.db $03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03
.db $04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04
.db $05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05
.db $06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06
.db $07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07
.db $08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08
.db $09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09
.db $0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A
.db $0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B
.db $0C,$0D,$0E,$0F,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C
.db $0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D
.db $0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E
.db $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00
.db $01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01
.db $02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02
.db $03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03
.db $04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04
.db $05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05
.db $06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06
.db $07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07
.db $08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08
.db $09,$0A,$0B,$0C,$0D,$0E,$0F,$01,$02,$03,$04,$05,$06,$07,$08,$09
.db $0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A
.db $0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B
.db $0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C
.db $0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D
.db $0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E
.db $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00
.db $01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01
.db $02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02
.db $03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03
.db $04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04
.db $05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05
.db $06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06
.db $07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07
.db $08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08
.db $09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09
.db $0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A
.db $0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B
.db $0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C
.db $0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D
.db $0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E
.db $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00
.db $01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01
.db $02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02
.db $03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F,$01,$02,$03
.db $04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04
.db $05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05
.db $06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06
.db $07,$08,$09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07
.db $08,$09,$0A,$0B,$0C,$0D,$0E,$0F,$01,$02,$03,$04,$05,$06,$07,$08
.db $09,$0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09
.db $0A,$0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A
.db $0B,$0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B
.db $0C,$0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C
.db $0D,$0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D
.db $0E,$00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E
.db $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
.db $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
.db $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
.db $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F

sms2ngp_freqhi_tbl:
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01
.db $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02,$02
.db $02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$03,$03,$03
.db $03,$03,$03,$03,$03,$03,$03,$03,$03,$03,$03,$03,$04,$04,$04,$04
.db $04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$05,$05,$05,$05,$05
.db $05,$05,$05,$05,$05,$05,$05,$05,$05,$05,$06,$06,$06,$06,$06,$06
.db $06,$06,$06,$06,$06,$06,$06,$06,$06,$07,$07,$07,$07,$07,$07,$07
.db $07,$07,$07,$07,$07,$07,$07,$07,$08,$08,$08,$08,$08,$08,$08,$08
.db $08,$08,$08,$08,$08,$08,$08,$09,$09,$09,$09,$09,$09,$09,$09,$09
.db $09,$09,$09,$09,$09,$09,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A
.db $0A,$0A,$0A,$0A,$0A,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B,$0B
.db $0B,$0B,$0B,$0B,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C
.db $0C,$0C,$0C,$0C,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
.db $0D,$0D,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E,$0E
.db $0E,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
.db $10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10,$11
.db $11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$11,$12,$12
.db $12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$12,$13,$13,$13
.db $13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$13,$14,$14,$14,$14
.db $14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$14,$15,$15,$15,$15,$15
.db $15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$16,$16,$16,$16,$16,$16
.db $16,$16,$16,$16,$16,$16,$16,$16,$16,$17,$17,$17,$17,$17,$17,$17
.db $17,$17,$17,$17,$17,$17,$17,$17,$18,$18,$18,$18,$18,$18,$18,$18
.db $18,$18,$18,$18,$18,$18,$18,$19,$19,$19,$19,$19,$19,$19,$19,$19
.db $19,$19,$19,$19,$19,$19,$19,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A,$1A
.db $1A,$1A,$1A,$1A,$1A,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B,$1B
.db $1B,$1B,$1B,$1B,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C
.db $1C,$1C,$1C,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D,$1D
.db $1D,$1D,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E,$1E
.db $1E,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F
.db $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$21
.db $21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$21,$22,$22
.db $22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$22,$23,$23,$23
.db $23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$23,$24,$24,$24,$24
.db $24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$24,$25,$25,$25,$25,$25
.db $25,$25,$25,$25,$25,$25,$25,$25,$25,$25,$26,$26,$26,$26,$26,$26
.db $26,$26,$26,$26,$26,$26,$26,$26,$26,$27,$27,$27,$27,$27,$27,$27
.db $27,$27,$27,$27,$27,$27,$27,$27,$28,$28,$28,$28,$28,$28,$28,$28
.db $28,$28,$28,$28,$28,$28,$28,$29,$29,$29,$29,$29,$29,$29,$29,$29
.db $29,$29,$29,$29,$29,$29,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A,$2A
.db $2A,$2A,$2A,$2A,$2A,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B,$2B
.db $2B,$2B,$2B,$2B,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C,$2C
.db $2C,$2C,$2C,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D,$2D
.db $2D,$2D,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E,$2E
.db $2E,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F,$2F
.db $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$30,$31
.db $31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$31,$32,$32
.db $32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$32,$33,$33,$33
.db $33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$33,$34,$34,$34
.db $34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$35,$35,$35,$35,$35
.db $35,$35,$35,$35,$35,$35,$35,$35,$35,$35,$36,$36,$36,$36,$36,$36
.db $36,$36,$36,$36,$36,$36,$36,$36,$36,$37,$37,$37,$37,$37,$37,$37
.db $37,$37,$37,$37,$37,$37,$37,$37,$38,$38,$38,$38,$38,$38,$38,$38
.db $38,$38,$38,$38,$38,$38,$38,$38,$39,$39,$39,$39,$39,$39,$39,$39
.db $39,$39,$39,$39,$39,$39,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A,$3A
.db $3A,$3A,$3A,$3A,$3A,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B
.db $3B,$3B,$3B,$3B,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C,$3C
.db $3C,$3C,$3C,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D
.db $3D,$3D,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E
.db $3E,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
.db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
.db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
.db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
.db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F


_vgm_player_end:
